@@ -27,6 +27,7 @@ from neutron_lib import constants
 from neutron_lib import context as neutron_context
 from neutron_lib import rpc as n_rpc
 from oslo_config import cfg
+from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 import oslo_messaging as messaging
 from oslo_service import service
@@ -37,15 +38,14 @@ LOG = logging.getLogger(__name__)
 class TaasPluginApi(api.TaasPluginApiMixin):
 
     def __init__(self, topic, host):
-        super(TaasPluginApi, self).__init__(topic, host)
+        super().__init__(topic, host)
         target = messaging.Target(topic=topic, version='1.0')
         self.client = n_rpc.get_client(target)
-        return
 
     def sync_tap_resources(self, sync_tap_res, host):
         """Send Rpc to plugin to recreate pre-existing tap resources."""
-        LOG.debug("In RPC Call for Sync Tap Resources: Host=%s, MSG=%s" %
-                  (host, sync_tap_res))
+        LOG.debug("In RPC Call for Sync Tap Resources: Host=%s, MSG=%s",
+                  host, sync_tap_res)
 
         context = neutron_context.get_admin_context()
 
@@ -53,12 +53,9 @@ class TaasPluginApi(api.TaasPluginApiMixin):
         cctxt.cast(context, 'sync_tap_resources', sync_tap_res=sync_tap_res,
                    host=host)
 
-        return
-
     def set_tap_service_status(self, msg, status, host):
         LOG.debug("In RPC Call for set tap service status: Host=%s, MSG=%s, "
-                  "Status=%s" %
-                  (host, msg, status))
+                  "Status=%s", host, msg, status)
 
         context = neutron_context.get_admin_context()
 
@@ -66,12 +63,9 @@ class TaasPluginApi(api.TaasPluginApiMixin):
         cctxt.cast(context, 'set_tap_service_status', msg=msg, status=status,
                    host=host)
 
-        return
-
     def set_tap_flow_status(self, msg, status, host):
         LOG.debug("In RPC Call for set tap flow status: Host=%s, MSG=%s, "
-                  "Status=%s" %
-                  (host, msg, status))
+                  "Status=%s", host, msg, status)
 
         context = neutron_context.get_admin_context()
 
@@ -79,7 +73,11 @@ class TaasPluginApi(api.TaasPluginApiMixin):
         cctxt.cast(context, 'set_tap_flow_status', msg=msg, status=status,
                    host=host)
 
-        return
+    @log_helpers.log_method_call
+    def set_tap_mirror_status(self, msg, status, host):
+        LOG.debug("In RPC Call for set Tap Mirror status")
+        # TODO(lajoskatona): We can have a status field to indicate that the
+        #  Neutron port is UP, is it useful?
 
 
 class TaasAgentRpcCallback(api.TaasAgentRpcCallbackMixin):
@@ -91,7 +89,7 @@ class TaasAgentRpcCallback(api.TaasAgentRpcCallbackMixin):
         self.conf = conf
         self.driver_type = driver_type
 
-        super(TaasAgentRpcCallback, self).__init__()
+        super().__init__()
 
     def initialize(self):
         self.taas_driver = manager.NeutronManager.load_class_for_provider(
@@ -119,6 +117,16 @@ class TaasAgentRpcCallback(api.TaasAgentRpcCallbackMixin):
                 'set_status_func_name': 'set_tap_flow_status',
                 'fail_status': constants.PENDING_DELETE,
                 'succ_status': constants.INACTIVE},
+            'create_tap_mirror': {
+                'msg_name': 'tap_mirror',
+                'set_status_func_name': 'set_tap_mirror_status',
+                'fail_status': constants.ERROR,
+                'succ_status': constants.ACTIVE},
+            'delete_tap_mirror': {
+                'msg_name': 'tap_mirror',
+                'set_status_func_name': 'set_tap_mirror_status',
+                'fail_status': constants.PENDING_DELETE,
+                'succ_status': constants.INACTIVE},
             'periodic_tasks': {
                 'msg_name': 'periodic_tasks',
             }
@@ -135,42 +143,45 @@ class TaasAgentRpcCallback(api.TaasAgentRpcCallbackMixin):
         LOG.debug("Invoking Driver for %(func_name)s from agent",
                   {'func_name': func_name})
 
+        status_msg = {}
         if func_name != 'periodic_tasks':
             func_dict = self.func_dict[func_name]
             status_msg = {'id': args[func_dict['msg_name']]['id']}
 
         try:
-            self.taas_driver.__getattribute__(func_name)(args)
+            driver_func = getattr(self.taas_driver, func_name)
+            driver_func(args)
         except Exception:
             LOG.error("Failed to invoke the driver")
-
-            self.taas_plugin_rpc.__getattribute__(
-                self.func_dict[func_name]['set_status_func_name'])(
-                    status_msg,
-                    self.func_dict[func_name]['fail_status'],
-                    self.conf.host)
+            rpc_func = getattr(
+                self.taas_plugin_rpc,
+                self.func_dict[func_name]['set_status_func_name'])
+            rpc_func(status_msg, self.func_dict[func_name]['fail_status'],
+                     self.conf.host)
             return
 
         if func_name != 'periodic_tasks':
-            self.taas_plugin_rpc.__getattribute__(
-                self.func_dict[func_name]['set_status_func_name'])(
-                    status_msg,
-                    self.func_dict[func_name]['succ_status'],
-                    self.conf.host)
+            rpc_func = getattr(
+                self.taas_plugin_rpc,
+                self.func_dict[func_name]['set_status_func_name'])
+            rpc_func(status_msg, self.func_dict[func_name]['succ_status'],
+                     self.conf.host)
 
-    def create_tap_service(self, context, tap_service, host):
+    def create_tap_service(self, context, tap_service_msg, host):
         """Handle Rpc from plugin to create a tap_service."""
-        if not self._driver_and_host_verification(host, tap_service['port']):
+        if not self._driver_and_host_verification(
+                host, tap_service_msg['port']):
             LOG.debug("RPC Call for Create Tap Serv. Either Host value [%s]"
                       "(received in RPC) doesn't match the host value "
                       "stored in agent [%s], or incompatible driver type. "
-                      "Ignoring the message." % (host, self.conf.host))
+                      "Ignoring the message.", host, self.conf.host)
             return
-        LOG.debug("In RPC Call for Create Tap Service: MSG=%s" % tap_service)
+        LOG.debug("In RPC Call for Create Tap Service: MSG=%s",
+                  tap_service_msg)
 
         return self._invoke_driver_for_plugin_api(
             context,
-            tap_service,
+            tap_service_msg,
             'create_tap_service')
 
     def create_tap_flow(self, context, tap_flow_msg, host):
@@ -178,30 +189,31 @@ class TaasAgentRpcCallback(api.TaasAgentRpcCallbackMixin):
             LOG.debug("RPC Call for Create Tap Flow. Either Host value [%s]"
                       "(received in RPC) doesn't match the host value "
                       "stored in agent [%s], or incompatible driver type. "
-                      "Ignoring the message." % (host, self.conf.host))
+                      "Ignoring the message.", host, self.conf.host)
             return
-        LOG.debug("In RPC Call for Create Tap Flow: MSG=%s" % tap_flow_msg)
+        LOG.debug("In RPC Call for Create Tap Flow: MSG=%s", tap_flow_msg)
 
         return self._invoke_driver_for_plugin_api(
             context,
             tap_flow_msg,
             'create_tap_flow')
 
-    def delete_tap_service(self, context, tap_service, host):
+    def delete_tap_service(self, context, tap_service_msg, host):
         #
         # Cleanup operations must be performed by all hosts
         # where the source and/or destination ports associated
         # with this tap service were residing.
         #
-        if not self._is_driver_port_type_compatible(tap_service['port']):
+        if not self._is_driver_port_type_compatible(tap_service_msg['port']):
             LOG.debug("RPC Call for Delete Tap Service. Incompatible driver "
-                      "type. Ignoring the message. Host=[%s]" % (host))
+                      "type. Ignoring the message. Host=[%s]", host)
             return
-        LOG.debug("In RPC Call for Delete Tap Service: MSG=%s" % tap_service)
+        LOG.debug("In RPC Call for Delete Tap Service: MSG=%s",
+                  tap_service_msg)
 
         return self._invoke_driver_for_plugin_api(
             context,
-            tap_service,
+            tap_service_msg,
             'delete_tap_service')
 
     def delete_tap_flow(self, context, tap_flow_msg, host):
@@ -209,15 +221,49 @@ class TaasAgentRpcCallback(api.TaasAgentRpcCallbackMixin):
             LOG.debug("RPC Call for Delete Tap Flow. Either Host value [%s]"
                       "(received in RPC) doesn't match the host value "
                       "stored in agent [%s], or incompatible driver type. "
-                      "Ignoring the message." % (host, self.conf.host))
+                      "Ignoring the message.", host, self.conf.host)
             return
-        LOG.debug("In RPC Call for Delete Tap Flow: MSG=%s" % tap_flow_msg)
+        LOG.debug("In RPC Call for Delete Tap Flow: MSG=%s", tap_flow_msg)
 
         return self._invoke_driver_for_plugin_api(
             context,
             tap_flow_msg,
             'delete_tap_flow')
 
+    @log_helpers.log_method_call
+    def create_tap_mirror(self, context, tap_mirror_msg, host):
+        """Handle Rpc from plugin to create a tap_mirror."""
+        if not self._driver_and_host_verification(
+                host, tap_mirror_msg['port']):
+            LOG.debug("RPC Call for Create Tap Mirror. Either Host value [%s]"
+                      "(received in RPC) doesn't match the host value "
+                      "stored in agent [%s], or incompatible driver type. "
+                      "Ignoring the message.", host, self.conf.host)
+            return
+        LOG.debug("In RPC Call for Creae Tap Mirror: MSG=%s", tap_mirror_msg)
+
+        return self._invoke_driver_for_plugin_api(
+            context,
+            tap_mirror_msg,
+            'create_tap_mirror')
+
+    @log_helpers.log_method_call
+    def delete_tap_mirror(self, context, tap_mirror_msg, host):
+        """...."""
+        if not self._driver_and_host_verification(host,
+                                                  tap_mirror_msg['port']):
+            LOG.debug("RPC Call for Delete Tap Mirror. Either Host value [%s]"
+                      "(received in RPC) doesn't match the host value "
+                      "stored in agent [%s], or incompatible driver type. "
+                      "Ignoring the message.", host, self.conf.host)
+            return
+        LOG.debug("In RPC Call for Delete Tap Mirror: MSG=%s", tap_mirror_msg)
+
+        return self._invoke_driver_for_plugin_api(
+            context,
+            tap_mirror_msg,
+            'delete_tap_mirror')
+
     def _taas_rpc_setup(self):
         # setup RPC to msg taas plugin
         self.taas_plugin_rpc = TaasPluginApi(
@@ -250,11 +296,11 @@ class TaasAgentRpcCallback(api.TaasAgentRpcCallbackMixin):
 
 class TaasAgentService(service.Service):
     def __init__(self, driver):
-        super(TaasAgentService, self).__init__()
+        super().__init__()
         self.driver = driver
 
     def start(self, taas_plugin_rpc, host):
-        super(TaasAgentService, self).start()
+        super().start()
 
         if self.driver.get_driver_type() == \
                 taas_ovs_consts.EXTENSION_DRIVER_TYPE:
